function x = Optimize(fun, model_parameters, lb, ub, type)
    % Optimize.m
    % This file implements two optimization methods  model impedance equation as described in
    % the document titled "EIS Model Fitting and Analysis". 
    %
    % Parameters 
    % @fun: Function handle to Objective function to be minimized.
    % @model_parameters: Initial vector of model parameters. 
    % @lb: Vector of lower bounds for model parameters. 
    % @ub: Vector of upper bounds for model parameters.
    % @type: indicates which optimization method to use (1-LM, 2-PSO).
    %
    % Author: Marvin Messing, CMHT, McMaster University
    % Date: July 26, 2017

    if type == 1
        %% LSQNONLIN
        % non-linear least squares optimization. 
        options = optimoptions('lsqnonlin',...
                                     'PlotFcns', 'optimplotx', ...
                                     'FunctionTolerance', 1e-16, ...
                                     'OptimalityTolerance', 1e-6, ...
                                     'MaxFunctionEvaluations', 1e10, ...
                                     'MaxIterations', 1e10, ...
                                     'StepTolerance', 1e-10, ...
                                     'Algorithm', 'levenberg-marquardt');

        x = lsqnonlin(fun, model_parameters, [], [], options);
    else
        %% PSO
        % particle-swarm optimization. 

        options = optimoptions('particleswarm',...
                                     'InitialSwarmMatrix', model_parameters, ...
                                     'PlotFcns', 'pswplotbestf', ...
                                     'SwarmSize', 100,...%2500, ...
                                     'MaxIterations', 5000, ...
                                     'FunctionTolerance', 1e-15, ...
                                     'ObjectiveLimit', 1e-10, ...
                                     'MaxStallIterations', 1000);

        x = particleswarm(fun, length(model_parameters), lb, ub, options);
    end
end

