function Z = RRCRC_MDL(f, model_parameters)
    % RRCRC_MDL.m
    % This file implements the R-RC-RC model impedance equation as described in
    % the document titled "EIS Model Fitting and Analysis". 
    %
    % Parameters 
    % @f: Vector of frequencies to model.
    % @model_parameters: Vector of model parameters. 
    %
    % Author: Marvin Messing, CMHT, McMaster University
    % Date: July 26, 2017

    L = model_parameters(1);
    R0 = model_parameters(2);
    R1 = model_parameters(3);
    C1 = model_parameters(4);
    R2 = model_parameters(5);
    C2 = model_parameters(6);

    s = size(f);
    w = 2*pi()*f;

    t1 = R1*C1;
    t2 = R2*C2;

    Z = 1j*w*L + R0*ones(s) + (R1*ones(s))./(1 + 1j*w*t1) + (R2*ones(s))./(1 + 1j*w*t2);

end

